/**
 * Architektur & Implementierung von DBMS, Aufgabenblatt 7
 *
 * Bitmap Indexing, Query Processing and WAH Compression
 *
 * File        : BitVector.h
 */

#ifndef BITVECTOR_H
#define BITVECTOR_H

#include "PlainBitVector.h"
#include "WAHBitVector.h"

/** 
 * Based on the make parameter WAH, we decide which one of the bitvector to
 * use. When WAH=1, we use WAHBitVector which does compression. Otherwise, we
 * use PlainBitVector which does not do any compression.
 */
#if defined(WAH) && WAH == 1 /* WAH */
typedef WAHBitVector BitVector;
#else /* PLAIN */
typedef PlainBitVector BitVector;
#endif

/** 
 * Efficient method for counting set bits on an unsigned integer. 
 * Taken from "Bit Twiddling Hacks":
 * http://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetParallel 
 * 
 * @param v 32-bit unsigned integer whose set bits will be counted
 * 
 * @return number of set bits
 */
inline
uint32_t bitcount(uint32_t v) {
    uint32_t c;

    v = v - ((v >> 1) & 0x55555555);                      // reuse input as temporary
    v = (v & 0x33333333) + ((v >> 2) & 0x33333333);       // temp
    c = (((v + (v >> 4)) & 0xF0F0F0F) * 0x1010101) >> 24; // count

    return c;
}

#endif /* BITVECTOR_H */
